//	ukasz Mirtecki, 3961/D, E1
//  l.mirtecki@elka.pw.edu.pl    
//	Algorytm : adjacent_difference (STL)

/*	adjacent_difference jest nazw przecionej funkcji wystpujcej w dwch postaciach :

	template <class InputIterator, class OutputIterator>
	OutputIterator adjacent_difference(InputIterator first, InputIterator last, 
                                   OutputIterator result);

	template <class InputIterator, class OutputIterator, class BinaryFunction>
	OutputIterator adjacent_difference(InputIterator first, InputIterator last,
                                   OutputIterator result,
                                   BinaryFunction binary_op);

	Funkcja adjacent_difference() w zakresie okrelonym przez iteratory //first//, //last// 
	dokonuje odejmowania kolejnych przylegajcych do siebie elementw. Element *first przypisywany 
	jest do elementu *result bdcym iteratorem na kolekcje wynikow. Dla kadego elementu iteratora 
	i w zakresie [first + 1, last), rnica *i oraz *(i - 1) przypisywana jest do *(result + (i - first)).
	Dziki temu, i oprcz skadowania rnic w kolekcji wynikowej na pierwszej pozycji przechowujemy 
	pierwszy element moemy odtworzy kolekcj wejciow na podstawie, ktrej powsta wynik. Operacj 
	odwrotn do adjacent_difference, ktr mona uy do tego celu jest jest partial_sum.
	Nie ma przeciwwskaza aby iterator wynikowy by tosamy z wejciowym umoliwiajc obliczanie 
	rnic na kolekcji 'w miejscu'.

	===== Wymagany nagwek =====
	#include<numeric>

	===== Parametry ===== 
	Wymagania dla obu wersji:
		first  - iterator wskazujcy pocztek zakresu wejciowego
		last   - iterator wskazujcy koniec zakresu wejciowego
		result - iterator wskazujcy pocztek kolekcji wyjciowej
		Wartoci dla typu InputIterator daj si konwertowa do wartoci dla typu OutputIterator
		[first, last) jest poprawnym zakresem 
		[result, result + (last - first)) jest poprawnym zakresem 

	Dodatkowe wymagania dla pierwszej wersji:
		Warto zwracana x - y jest zdefiniowana i daje si konwertowa do wartoci OutputIterator. 

	Dodatkowe wymagania dla drugiej wersji:
		BinaryFunction jest funkcj binarn.
		Wartoci dla typu InputIterator daj si konwertowa odpowiednio do wartoci pierwszego i drugiego argumentu funkcji binarnej. 
		Wynik funkcji binarnej daj si konwertowa do wartoci dla typu OutputIterator. 
			
	===== Zoono =====
	Liniowa. brak operacji jeli [first, last) jest przedziaem pustym, w przeciwnym przypadku dokadnie (last - first) - 1 operacji. 
*/


//#include "stdafx.h"

// Nagwek gwny
#include <numeric>

// Nagwki pomocnicze dla przykadu
#include <vector>
#include <list>
#include <functional>
#include <iostream>

using namespace std;

int _tmain(int argc, _TCHAR* argv[])
{
   list <int> L1; // Lista wejciowa
   list <int>::iterator LIter1, LIterend, LIterend2;

   vector<int> V1( 10 ), V2( 10 ); // Kontenery wyjciowe
   vector<int>::iterator VIter1, VIter2, VIterend, VIterend2;

   int t; // Wypenienie  kolekcji wejciowej {1, 4, 9, 16, 25, 36, 49, 64, 81, 100};
   for ( t = 1 ; t <= 10 ; t++ )
      L1.push_back( t * t );

   cout << "Kolekcja wejsciowa L1 :\n ( " ;
   for ( LIter1 = L1.begin( ) ; LIter1 != L1.end( ) ; ++LIter1 )
      cout << *LIter1 << " ";
   cout << ")." << endl;

   //  Zastosowanie pierwszej wersji algorytmu
   //  Standardowa uycie adjacent_difference na licie, wynik zwracany przypisany do wektora
   VIterend = adjacent_difference ( L1.begin ( ) , L1.end ( ) , V1.begin ( ) );
   
   cout << "Kolekcja wyjsciowa, standardowe adjacent_differences:\n ( " ;
   for ( VIter1 = V1.begin( ) ; VIter1 != VIterend ; ++VIter1 )
      cout << *VIter1 << " ";
   cout << ")." << endl;

   //  Zastosowanie drugiej wersji algorytmu
   //  Uycie adjacent_difference z funkcj binarn - mnoenie - na licie, wynik zwracany przypisany do wektora
   VIterend2 = adjacent_difference ( L1.begin ( ) , L1.end ( ) , V2.begin ( ) , 
      multiplies<int>( ) );
   
   cout << "Kolekcja wyjsciowa, adjacent_differences dla funcji binarnej - mnozenia:\n ( " ;
   for ( VIter2 = V2.begin( ) ; VIter2 != VIterend2 ; ++VIter2 )
      cout << *VIter2 << " ";
   cout << ")." << endl;

   // Zastosowanie algorytmu 'w miejscu'
   LIterend2 = adjacent_difference ( L1.begin ( ) , L1.end ( ) , L1.begin ( ) );
   cout << "Kolekcja wyjsciowa i wejsciowa tozsame, adjacent_differences w miesjcu na L1:\n ( " ;
   for ( LIter1 = L1.begin( ) ; LIter1 != LIterend2 ; ++LIter1 )
      cout << *LIter1 << " ";
   cout << ")." << endl;


	//-----------------------------------------------------------------------------
	//  Zastosowanie algorytmu 'w rzeczywistym' przypadku
	typedef vector < int > IntegerArray;
	typedef ostream_iterator < int, char, char_traits<char> > IntOstreamIt;

	// iterator ostream drukujcy liczny cakowite int zakoczone znakiem spacji
    IntOstreamIt itOstream(cout," ");

    // Inicjalizacja tablic
    // Sytaucja :
	//			Podrujc jestemy w stanie mierzy odlegoci
	//			pomidzy miastem wyjazdu a miastem w ktrym aktualnie
	//			si znajdujemy (np. licznik w samolocie)
	IntegerArray rgIA;
    rgIA.push_back(5661); //  San Francisco - Berlin
    rgIA.push_back(7456); //  Kair
    rgIA.push_back(10995); // Kalkuta
    rgIA.push_back(17019); // Cape Town
    rgIA.push_back(24394); // Hong Kong
    rgIA.push_back(30376); // Londyn
    rgIA.push_back(35758); // Los Angeles

    // Wydruk tablicy wejciowej
    copy(rgIA.begin(),rgIA.end(),itOstream);
    cout << endl;

    // Chcemy znale odlegoci jakie musielimy przeby
	// miedzy poszczeglnymi miastami
	// Wykorzystujemy  adjacent_difference()
    IntegerArray rgDifferences(7);
    IntegerArray::iterator itDifferences = rgDifferences.begin();
    adjacent_difference(rgIA.begin(),rgIA.end(),itDifferences); // Oblicznie roznic

    // Wydruk 'odlegoci' pomidzy miastami
    // Naley pamita ze pierwszy element nie jest rnica !
    cout << "Kolejne roznice pomiedzy miastami: ";
    copy(rgDifferences.begin()+1,rgDifferences.end(),itOstream);
    cout << endl;
}


